
/*  coalescence on a fixed pedigree  */
/*  builds Wright-Fisher pedigrees and gene genealogies within them  */
/*  can make pseudo-data by putting mutations on the gene genealogy  */

/*  this is a bare-bones version of the code used in  */
/*  Wakeley, King, Low, and Ramachandran (2012, Genetics, Vol. 190, 1433–1445)  */
    /* as is, it just makes one pedigree then follows samplesize=10 */
    /* genetic lineages from samplesize=10 individuals, building the gene genealogy */ 
    /* numreps=100000 times, and calculates various statistics  */ 
    /* this takes about 785MB of RAM and about 1.5 minutes on my 2.4GHz Mac laptop */
    /* modify as you like, and please cite the paper above */
    /* send questions to me at wakeley@fas.harvard.edu */ 
    /* please excuse the lingering comments to self */


#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <math.h>
#include <time.h>

/* these are the nodes of the coalescent trees; developed in /Cprograms/NeutralityTests/coalescetestsnew.c */

struct node {
	int id;  /* 0 through n-1 are the sample/tips, and n to 2n-2 are the internal nodes (n is called samplesize below). */
	double time;  /* time here will be measured in generations */ 
	int parenttofollow;  /* the id of either the mother or the father (depending on what pormchromosome was in the previous generation. */ 
	int chromosomeinparent;  /* 0 or 1 depending on whether you're going to follow the maternal of paternal chromosome. */
	struct node *desc1;  
	struct node *desc2;
};

/* global variables(!) */ 
/*  note that you've decided to measure time backwards starting here,  */ 
/*  but the pedigree will generally be constructed forward in time.    */

typedef int twoints[2]; /* makes allocating the 3D pedigree (numgens by N [which may vary with time] by 2) easier */
twoints **pedigree;  	/* pedigree[i][j][0] will be the index of the female parent of jth individual in the ith generation in the past */
			/* pedigree[i][j][1] will be the index of the male parent of jth individual in the ith generation in the past */
twoints *popsizes;   /* popsizes[i][0] will be the number of females in the population in the ith generation in the past */
                     /* popsizes[i][1] will be the number of males in the population in the ith generation in the past */
size_t pedigree_alloc_counter,pedigree_free_counter;  /* these four are to make sure memory allocated = memory freed */
size_t popsizes_alloc_counter,popsizes_free_counter;  /* see AllocateTopLevel, FreeAllMemory, PrintMemoryUsage */
						      /* and MakeWrightFisherPedigree, and other MakePedigrees   */
/* function prototypes */

void MyError(char[]);
void WriteTime(long,long,FILE*);
void PrintMemoryUsage(FILE*);
double unirandom(void);
double exponentialdev(void);
double localgammln(double);
double poissondev(double);
int UniformRandomInteger(int,int);
void AllocateTopLevel(int);
void FreeAllMemory(int);
void SetPopSizesAllocatePedigree(int,int,double);
void MakeWrightFisherPedigree(int);
void MakeCyclicalWrightFisherPedigree(int);
void PrintPedigreeStuff(int,FILE*);
void SampleWithoutReplacement(int,int,int,int*);
/* the ones below are the routines you incorporated from /Cprograms/NeutralityTests/coalescetestsnew.c */
void InitializeTree(int,struct node*);
void CoalescePair2(int,int,int*,struct node*,struct node**,double);
void InitializeTreeTips(int,int*,struct node*);
void FollowLineagesMakeTree(int,int,struct node*);
double CalculateTotalTreeLength(int,struct node*);
int PickBranch(int,double,struct node*);
void Mutate(int,int,char**,struct node*);
void MakeData(int,int,char**,double,struct node*);
void PrintData(int,int,char**,FILE*);
void WriteTreeInfo(int,struct node*,FILE*);
void WriteParenthetical(struct node,int,FILE*);
int Nchoose2(int);
void CountFreqs(int,int,int*,char**,double*);
void PrintStats(int,int,int,double,double*,double*,double*,FILE*);

/* MyError - simple error report and exit */ 

void MyError(error_text)
char  error_text[];
{
	fprintf(stderr,"\nJW's run-time error...\n");
	fprintf(stderr,"\n\t%s\n",error_text);
	fprintf(stderr,"\n...now exiting to system...\n\n");
	exit(1);
}

/* WriteTime - takes start and finish, in seconds, and prints hour:mins:sec to file */

void WriteTime(start,finish,fptr)
long start,finish;
FILE *fptr;
{
	double seconds,hours,minutes;
	
	seconds = (double)(finish-start);
	modf(seconds/3600.0,&hours);
	seconds -= 3600.0*hours;
	modf(seconds/60.0,&minutes);
	seconds -= 60.0*minutes;
	fprintf(fptr,"\n Total Running Time = %1.0f:%1.0f:%1.0f\n\n",hours,minutes,seconds);
	fflush(fptr);
}

/* PrintMemoryUsage - prints the values of the global _alloc_ and _free_ counters above to file */

void PrintMemoryUsage(fptr)
FILE *fptr;
{
	/* fprintf(fptr,"\npedigree_alloc_counter = %u",(unsigned)pedigree_alloc_counter);
	fprintf(fptr,"\npedigree_free_counter  = %u\n",(unsigned)pedigree_free_counter);
	fprintf(fptr,"\npopsizes_alloc_counter = %u",(unsigned)popsizes_alloc_counter);
	fprintf(fptr,"\npopsizes_free_counter  = %u\n",(unsigned)popsizes_free_counter); */
	fprintf(fptr,"\n\ttotal pedigree memory usage in MB = %f\n\n",(double)(pedigree_alloc_counter+popsizes_alloc_counter)/1048576.0);
	
	fflush(fptr);
}

/*	unirandom - uniform [0,1) random deviates.	*/

#define IM 2147483648.0

double unirandom(void)
{
	return (double)random()/IM; 
}

#undef IM

/*	exponentialdev - mean 1, deviates. For mean m, multiple by m.	*/

double exponentialdev(void) 
{ 
	return -log(1.0 - unirandom()); 
} 

/*	localgammln - natural log of the gamma function.	*/

double localgammln(xx)
double xx;
{
	double x,y,tmp,ser;
	static double cof[6]={76.18009172947146,-86.50532032941677,
		24.01409824083091,-1.231739572450155,
		0.1208650973866179e-2,-0.5395239384953e-5};
	int j;

	y=x=xx;
	tmp=x+5.5;
	tmp -= (x+0.5)*log(tmp);
	ser=1.000000000190015;
	for (j=0;j<=5;j++) ser += cof[j]/++y;
	return -tmp+log(2.5066282746310005*ser/x);
}

/*	poissondev - poisson deviates with mean xm.	*/

#define PI 3.141592653589793 
 
double poissondev(xm) 
double xm;
{ 
	static double sq,alxm,g,oldm=(-1.0); 
	double em,t,y; 
 
	if (xm < 12.0) { 
		if (xm != oldm) { 
			oldm=xm; 
			g=exp(-xm); 
		} 
		em = -1; 
		t=1.0; 
		do { 
			++em; 
			t *= unirandom(); 
		} while (t > g); 
	} else { 
		if (xm != oldm) { 
			oldm=xm; 
			sq=sqrt(2.0*xm); 
			alxm=log(xm); 
			g=xm*alxm-localgammln(xm+1.0); 
		} 
		do { 
			do { 
				y=tan(PI*unirandom()); 
				em=sq*y+xm; 
			} while (em < 0.0); 
			em=floor(em); 
			t=0.9*(1.0+y*y)*exp(em*alxm-localgammln(em+1.0)-g); 
		} while (unirandom() > t); 
	} 
	return em; 
} 

#undef PI 

/* UniformRandomInteger - returns an integer <=min and >=max, somewhat sloppily (i.e. w/out proper casting, use of floor, etc.) */

int UniformRandomInteger(min,max) 
int min,max;
{
	if( min > max ) MyError("min > max in UniformRandomInteger");
	return ( min + (max-min+1)*unirandom() );
}

/*  AllocateTopLevel - allocates the memory for pointers to numgens generations of pedigree, */ 
/*  as well as entire popsizes array (fixed to numgens by 2); also keeps track of memory */

void AllocateTopLevel(numgens)
int numgens;
{
	int x;
	
	/* set memory counters to zero */
	pedigree_alloc_counter = 0;
	pedigree_free_counter = 0;
	popsizes_alloc_counter = 0;
	popsizes_free_counter = 0;
		
	/* allocate top level of pedigree memory */
	pedigree = (twoints **)malloc( (size_t)numgens*sizeof(twoints *) );
	if( pedigree == NULL ) MyError("problem with pedigree allocation in AllocateTopLevel");
	pedigree_alloc_counter += (size_t)numgens*sizeof(twoints *);
	
	/* allocate popsize memory */
	popsizes = (twoints *)malloc( (size_t)numgens*sizeof(twoints) );
	if( popsizes == NULL ) MyError("problem with popsizes allocation in AllocateTopLevel");
	popsizes_alloc_counter += (size_t)numgens*sizeof(twoints);
	
	for(x=0;x<numgens;x++) {  /* added this as a flag when you wrote SampleWithoutReplacement */
		popsizes[x][0] = -1;
		popsizes[x][1] = -1;
	}
}

/*  FreeAllMemory - frees the pedigree and popsizes memory allocated earlier during the run */

void FreeAllMemory(numgens)
int numgens;
{
	int x;
	
	/* free lower level of pedigree memory */
	for(x=0;x<numgens;x++) {
		pedigree_free_counter += (size_t)(popsizes[x][0]+popsizes[x][1])*sizeof(twoints);
		free( (void *)pedigree[x] );
	} 

	/* free memory of highest level pedigree and popsizes pointers */
	free( (void *)pedigree ); 
	pedigree_free_counter += (size_t)numgens*sizeof(twoints *);
	free( (void *)popsizes ); 
	popsizes_free_counter += (size_t)numgens*sizeof(twoints);
}

/* SetPopSizesAllocatePedigree - this routine just does the popsize memory allocation, setting popszies, and allocating lower */
/* level of pedigree memory, so that this can be done once (and then use MakeWrightFisherPedigreeNew to make each pedigree).  */ 
/* certainly makes sense when Nf and Nm are fixed, but you'll have to think more if you use it for variable Nf and Nm */

void SetPopSizesAllocatePedigree(numgens,N,f)
int numgens,N;
double f;
{
	int x;
	
	/* the pedigree is constructed forward in time, hence the x-- in this for loop */
	for(x=numgens-1;x>=0;x--) {
	
		/* set the poplation sizes */
		/* with fixed number of females and males */
		popsizes[x][0] = (int)( f*(double)N );    
		popsizes[x][1] = N - popsizes[x][0];  
		
		/* or (one possible way) with random numbers of males and females, semi-mimicking hemizygosity */
		/* popsizes[x][0] = 1 + (int)poissondev( 0.5*(double)N - 1.0 ); 
		popsizes[x][1] = 1 + (int)poissondev( 0.5*(double)N - 1.0 ); */
		/* allocate lower level of pedigree memory */
		
		pedigree[x] = (twoints *)malloc( (size_t)(popsizes[x][0]+popsizes[x][1])*sizeof(twoints) );
		if( pedigree[x] == NULL ) MyError("problem with pedigree[x] allocation in SetPopSizesAllocatePedigree"); 
		pedigree_alloc_counter += (size_t)(popsizes[x][0]+popsizes[x][1])*sizeof(twoints);
	}
}

/* MakeWrightFisherPedigree - constructs a pedigree with numgens generations with (WF) random mating   */
/* between some numbers of males and females, either fixed or varying simply (at the moment Poisson).  */
/* YOU MUST CALL SetPopSizesAllocatePedigree BEFORE USING THIS FUNCTION */
/* Note that the individuals in the top generation, numgens-1, are assigned "parents"  */
/* which are used for cycling back through the pedigree if Tmrca has not been reached. */

void MakeWrightFisherPedigree(numgens)
int numgens;
{
	int x,y;
    int loopgen;
    
    loopgen = 0;  /* change if you'd ike to loop back to some other generation (when the gene genealogy exceeds numgens in heigth) */
	
	/* the pedigree is constructed forward in time, hence the x-- in the for loop down below */
	x = numgens-1; /* this y for loop is just for this case, when you're at the top of the pedigree and need to loop back down */
    for(y=0;y<popsizes[x][0]+popsizes[x][1];y++) {    /* for each member of the population, female or male, do WF sampling of parents */
        pedigree[x][y][0] = UniformRandomInteger(0,popsizes[loopgen][0]-1);  /* pick female parent uniformly at random from generation 0 */
        if( pedigree[x][y][0] < 0 || pedigree[x][y][0] > popsizes[loopgen][0]-1 ) MyError("what the heck 1");
        pedigree[x][y][1] = UniformRandomInteger(popsizes[loopgen][0],popsizes[loopgen][0]+popsizes[loopgen][1]-1);  /* pick male parent uniformly at random from generation 0 */
        if( pedigree[x][y][1] < popsizes[loopgen][0] || pedigree[x][y][1] > popsizes[loopgen][0]+popsizes[loopgen][1]-1 ) MyError("what the heck 2");
    }
	for(x=numgens-2;x>=0;x--) 
		for(y=0;y<popsizes[x][0]+popsizes[x][1];y++) {    /* for each member of the population, female or male, do WF sampling of parents */
			pedigree[x][y][0] = UniformRandomInteger(0,popsizes[x+1][0]-1);  /* pick female parent uniformly at random from the previous generation */
			if( pedigree[x][y][0] < 0 || pedigree[x][y][0] > popsizes[x+1][0]-1 ) MyError("what the heck 1");
			pedigree[x][y][1] = UniformRandomInteger(popsizes[x+1][0],popsizes[x+1][0]+popsizes[x+1][1]-1);  /* pick male parent uniformly at random from the previous generation */
			if( pedigree[x][y][1] < popsizes[x+1][0] || pedigree[x][y][1] > popsizes[x+1][0]+popsizes[x+1][1]-1 ) MyError("what the heck 2");
		}
}

/* MakeCyclicalWrightFisherPedigree - makes one generation of a WF pedigree, */
/* but then repeats that pattern exactly for every other past generation.    */ 
/* This will only work if popsizes are fized for all time. */
/* YOU MUST CALL SetPopSizesAllocatePedigree BEFORE USING THIS FUNCTION */ 
/* Note, this is very wastful because you really only need a one-generation-tall pedigree in this case.... */

void MakeCyclicalWrightFisherPedigree(numgens)
int numgens;
{
	int x,y;
	
	/* the pedigree is constructed forward in time, hence the x-- in this for loop */
	for(x=numgens-1;x>=0;x--) 
		if(x == numgens-1)  /* only do WF sampling for the 1st (top) generation */
			for(y=0;y<popsizes[x][0]+popsizes[x][1];y++) {    /* for each member of the population, female or male, do WF sampling of parents */
				pedigree[x][y][0] = UniformRandomInteger(0,popsizes[x][0]-1);  /* pick female parent unifrormly at random */
				if( pedigree[x][y][0] < 0 || pedigree[x][y][0] > popsizes[x][0]-1 ) MyError("what the heck 1");
				pedigree[x][y][1] = UniformRandomInteger(popsizes[x][0],popsizes[x][0]+popsizes[x][1]-1);  /* pick male parent unifrormly at random */
				if( pedigree[x][y][1] < popsizes[x][0] || pedigree[x][y][1] > popsizes[x][0]+popsizes[x][1]-1 ) MyError("what the heck 2");
			}
		else   /* all other generations, from numgens-2 down to the present generation 0 */
			for(y=0;y<popsizes[x][0]+popsizes[x][1];y++) {    /* for each member of the population, copy relationships of the generation above */
				pedigree[x][y][0] = pedigree[x+1][y][0];  
				pedigree[x][y][1] = pedigree[x+1][y][1];  
			}
}

void PrintPedigreeStuff(numgens,fptr)
int numgens;
FILE *fptr;
{
	int x,y;
	
	/* print out list of population sizes */
	for(x=0;x<numgens;x++) 
		fprintf(fptr,"%d\t%d\n",popsizes[x][0],popsizes[x][1]); 
	
	/* print out pedigree */
	for(x=numgens-1;x>=0;x--)
		for(y=0;y<popsizes[x][0]+popsizes[x][1];y++) 
			if(y == popsizes[x][0]+popsizes[x][1]-1)
				fprintf(fptr,"(%d,%d)\n",pedigree[x][y][0],pedigree[x][y][1]);
			else
				fprintf(fptr,"(%d,%d)  ",pedigree[x][y][0],pedigree[x][y][1]); 
}

/* SampleWithoutReplacement - written with small samples (from large populations in mind). */
/* it solves the problem that you need to skip over the integers you've already sampled by */
/* keeping the sampled numbers in an array, ordered from smallest to largest. each number  */
/* that's chosen scoots up this list, incrementing itself every time it hits a number that */
/* it's >=, until it finds a number it's <. then all the bigger numbers are moved over to  */
/* make a place for the new one. you first wrote this to sample from 0 to populationsize-1 */ 
/* then later switch to (min,max), so maybe should rewrite.... also, don't use this if the */
/* sample size is greater than populationsize/2 because then (using this routine) it would */
/* be faster to sample the numbers you don't want, and keep the unsampled ones.... */ 
/* REMEMBER THAT THE LIST OF SAMPLES IS ORDERED!! */ 
/* Note also that this differs slightly from the routine with the same name (!) in the */
/* programs pedigreecoal.c, and pedigreecoal2.c, and even pedigreecoaln24July2010AM.c  */
/* because you got rid of the testing functionality. */

void SampleWithoutReplacement(min,max,samplesize,samples)
int min,max;       /* range from which to draw: the integers from min to max, inclusive */
int samplesize;    /* number of samples to take */ 
int *samples;      /* array which will hold the sampled numbers, in order from smallest to largest */
{
	int x,y;  
	int populationsize;  /* total population size to sample from, which is number of the integers from min to max, inclusive */
	int pick,place;      /* dummy variables used in choosing w/out replacement */ 
	
	populationsize = max-min+1;
	if(populationsize <= 0 || populationsize < samplesize) MyError("bad population size in SampleWithoutReplacement");
	if(populationsize==samplesize) 
		for(x=0;x<samplesize;x++)    /* special case of sampling whole population  */
			samples[x] = x;       /* sample all of (0,populationsize-1)=(0,samplesize-1) */
	else {
		place = -1;  /* crazy value to make sure this gets set below; see line after for loop */
		for(x=0;x<samplesize;x++) {   /* for the number of samples you want to take */
			samples[x] = populationsize;   /* "buffer" value needed below in ordering the samples */
							/* also a check to make sure samples[x] gets a good value in the end */
			pick = UniformRandomInteger(0,populationsize-1-x);  /* pick a uniform random whole number between 0 and populationsize-1-x, inclusive. the  */
							    	/* -x is for sampling without replacement (each time, must decrease sample size by one) */
			for(y=0;y<=x;y++)  /* for each sample taken so far, including the buffer value */
				if(samples[y] <= pick)  /* each time samples[y] is less than the number you picked  */
					pick += 1;  /* increment your pick to account for the fact that, while you've randomly sampled from 0 and populationsize-1-x, */
				else {	/* what you really want to be doing is sample uniformly among the populationsize-x numbers that have not yet been chosen. */
					place = y;   /* otherwise, if pick<samples[y], then you've found the place your pick should be in the samples array, */
					y = x+1;     /* then exist this crazy for loop. */
				}
			for(y=x;y>place;y--) 
				samples[y] = samples[y-1];   /* scoot all the samples larger than your modified pick over to make room in place */
			samples[place] = pick;           /* then put your modified pick in its place, so the samples in the array are in increasing order. */ 
			if(samples[x] > populationsize-1) MyError("bad pick in SampleWithoutReplacement");  /* check referred to in comment to first line of loop */
		}
		if(place == -1) MyError("something seriously odd with place in SampleWithoutReplacement");
	}
	for(x=0;x<samplesize;x++)    /* for every sampled number, add the minimum possible value; */
		samples[x] += min;       /* changes the range from (0,populationsize-1) to (min,max). */
}

/* HEY: the routines below are from another program of your, /Cprograms/NeutralityTests/coalescetestsnew.c */
/* They are not commented so well, except for the new ones or ones you've modified. */

void InitializeTree(samplesize,tree)
int samplesize;
struct node *tree;
{
	int x;

	for(x=0;x<(2*samplesize);x++) {
		(tree + x)->id = x;
		(tree + x)->time = 0.0;
		(tree + x)->parenttofollow = -1-x; /* unique nonsense value */
		(tree + x)->chromosomeinparent = -1; /* unique nonsense value */
		(tree + x)->desc1 = NULL;
		(tree + x)->desc2 = NULL;
	}
}

/* CoalescePair2 - the same basic steps as in Hudson's MakeTree, but using a pre-chosen pair of lineages  */
/* rather than making two successive picks.  The pair must be in order so pairtocoal[1] > pairtocoal[0].  */ 
/* This one differs from plain CoalescePair in that it sets parenttofollow and chromosomeinparent for the */
/* new node to be equal to that of its descendents. */

void CoalescePair2(n,coalnode,pairtocoal,tree,list,generation)
int n;
int coalnode;
int pairtocoal[2];
struct node *tree;
struct node **list;
double generation;
{
	if(pairtocoal[0]>=pairtocoal[1]) MyError("pairs must be correctly ordered in CoalescePair");
	/* First two lines are easy: assign the two descendents. */ 
	tree[coalnode].desc1 = list[pairtocoal[1]];  /* Make 1st descendent of new node equal to member of coal pair with larger index in list.  */
	tree[coalnode].desc2 = list[pairtocoal[0]];  /* Make 2nd descendent of new node equal to member of coal pair with smaller index in list. */
	/* Next two lines are tricky: make swaps so that list is repacked for the next go 'round, when n will be one smaller. */ 
	/* You no longer want the two nodes that coalesced, but you want the new node in the list */
	list[pairtocoal[1]] = list[n-1];	    /* Swap larger-index node of the coal-pair in the list for the last element of the list. */
	list[pairtocoal[0]] = tree + coalnode;  /* Swap smaller-index node of the coal-pair in the list for the new node. */
	/* Next three lines make further assignments to the new node */ 
	tree[coalnode].time = generation;  /* set the time of the new node to the current generation.  */
	tree[coalnode].parenttofollow = tree[coalnode].desc1->parenttofollow;  /* Give new node same parent in pedigree as its descdent node(s). */
	tree[coalnode].chromosomeinparent = tree[coalnode].desc1->chromosomeinparent;  /* Also give it the same chromosome as its descdent node(s). */
}

/* InitializeTreeTips - initialize the tips of the tree, prior to entering FollowLineagesMakeTree. */

void InitializeTreeTips(samplesize,sampleids,tree)
int samplesize;
int *sampleids;
struct node *tree;
{
	int x;
	int parent; 

	for(x=0;x<samplesize;x++) {
		(tree + x)->time = 0.0;  /* should already have done this in InitializeTree, but jsut in case.... */
		parent = UniformRandomInteger(0,1);   /* 50:50 choice between mother and father */
		(tree + x)->parenttofollow =  pedigree[0][sampleids[x]][parent];  /* parent to follow is the mother or father of individual sampleids[x] */
		(tree + x)->chromosomeinparent = UniformRandomInteger(0,1);  /* the chromosome to follow in next step is maternal or paternal, randomly */
	} 
}
	
/* FollowLineagesMakeTree is built upon MakeTree2 in /Cprograms/NeutralityTests/coalescetestsnew.c  */
	/* comments from that program: */
/* MakeTree2 - this is a routine on the way to development of your pedigree-coalescence algorithm.  */
/* The basic idea is that the information in the nodes (parenttofollow and chromosomeinparent) will */
/* tell which nodes need to coalesce (all that have the same values of both).  The probelm is that  */
/* you need to look at this information to know which nodes to coalesce.  You opted to use Hudson's */
/* basic binary tree structure because you already had the code to creat pseudo-data sets for these */ 
/* trees. This might not be the best idea because you need to allow for the possibility of multiple */ 
/* coalescent events in the same generation; these trees are binary but might have some zero branch */
/* lengths. Your provisional solution is to use the double-for-loop construction below, which looks */ 
/* at every pair of lineages, then if it finds one does the coalescent event, assiging everything,  */ 
/* then starts over, looking at all pairs again, but now among n-1 instead of n lineages. This does */
/* what it needs to, but is inefficient:  (1) if there is only one pair to coalesce, which will be  */
/* most of the time, it has to do the double for loop twice: once to find the pair, and then again  */
/* to get out of the loop; and (2) if there are lots of simulataneous coalescent events, it would   */
/* probably be better to use the kind of trees in your exactlike.c program, in which the nodes can  */
/* have more than two descendants; that would have its own issues to figure out, though, due to the */
/* non-exchangeability of the condition pedigree-coal process compared with that simple balls-and-  */
/* boxes scheme (where you simply created lists of balls in boxes and randomly assigned lineages to */
/* them).  The stuff below might also have some merits of its own(?).  */
	/* new comments since adapting for use here: */
/* For a sample of two, this runs  just as fast as the old Coalesce2.  It also matches the exact */ 
/* expectations and variances in fiddles.nb perfectly for small populations (e.g. N=2), where it */
/* is possible to rebuild the pedigree for each replicate (these are expectations and variances  */
/* for the time to the common ancestor for samples of size 2); Coalesce2 matches these as well.  */

void FollowLineagesMakeTree(numgens,samplesize,tree)
int numgens;
int samplesize;
struct node *tree;
{
	int x,y;  /* indices for for loops */
	int n;  /* the number of lineages yer following; starts at samplesize and goes down to 1 (the mrca). */
	int coalnode;  /* the index of the next available note, which will be the common ancestral lineage of two that coalesce. */
	struct node **list;  /* an array of pointers to nodes */ 
	int pairtocoal[2]; /* you'll take a new pair each time you loop through the pedigree */ 
	int genwithin;   /* counter for generation number within pedgigree */
	int gentotal;    /* counter for the total number of generations since the present generation zero */
		
	list = (struct node **)malloc( (size_t)samplesize*sizeof(struct node *) );
	if(list == NULL) MyError("problem with list memory allocation");
	
	for(n=0;n<samplesize;n++) {
		if( (tree + n)->parenttofollow < 0 ) MyError("must call InitializeTreeTips before FollowLoneagesMakeTree");
		list[n] = tree + n;  /* initialize so that list[n] points to tree[n] */
	}

	gentotal = 0;  /* 0 is the present generation */
	genwithin = 0; /* ditto */
	n = samplesize;  /* set n to samplesize; n will go from samplesize down to 1 as coalescent events occur. */ 
	coalnode = samplesize;  /* set coalnode to samplesize; coalnode will go from samplesize up to 2*samplesize-2 as coalescent events occur. */
	while(n > 1) {  /* keep doing this until the most recent common ancestor is reached (n=1). */
		genwithin += 1; /* increment the current generation because now you're considering the parents. */
		gentotal += 1;  /* ditto for total elapsed time */
		if(genwithin==numgens) /* if you're dealing with parents who would be in generation numgens , */
			genwithin = 0;  /* then loop down to the bottom of the pedigree, and keep going */
		/* below is a kind of dangerous double for loop, in which x, y, and n all change throughout... */
		for(x=0;x<n-1;x++) /* x and y loop consider all pairs of active nodes in list; */
			for(y=x+1;y<n;y++) /* the if statement below checks to see if both came from the same parent and the same chromosome in the previous generation */
				if( list[x]->parenttofollow == list[y]->parenttofollow && list[x]->chromosomeinparent == list[y]->chromosomeinparent ) {
					pairtocoal[0] = x; /* these these are the pair to coalesce in this step; */
					pairtocoal[1] = y; /* and note that (pairtocoal[1] = y) > (pairtocoal[0] = x), as needed in CoalescePair2 */
					CoalescePair2(n,coalnode,pairtocoal,tree,list,(double)gentotal);  /* do the coalescent event */
					n -= 1;  /* decrease n by one */ 
					coalnode += 1;  /* increase coalnode by one, so that tree[coalnode] refers to the next unused node in the tree. */
					y = n;  /* you found a pair to coalesce, so get out of the y loop. */
					x = -1;  /* since there might be more pairs to coalesce, reset x so it will be 0 in the next run through double for loop */
				} 
		/* once you've done all your coalescing, the information in the active nodes needs to be updated, following lineages back one generation */
		for(x=0;x<n;x++) { /* next line: the node's new parenttofollow is its parent's mother or father, depending on chromosomeinparent, looping if necessary */
			list[x]->parenttofollow = pedigree[genwithin][list[x]->parenttofollow][list[x]->chromosomeinparent]; 
			list[x]->chromosomeinparent = UniformRandomInteger(0,1);  /* 50:50 choice whether to go maternal (0) or paternal (1) in next round. */
		} 
	}
	
	free( (void *)list );
}

double CalculateTotalTreeLength(samplesize,tree)
int samplesize;
struct node *tree;
{
	int x;
	double sum;

	sum = 0.0;
	for(x=(2*samplesize-2);x>=samplesize;x--) {
		sum += (tree[x].time - (tree+x)->desc1->time); 
		sum += (tree[x].time - (tree+x)->desc2->time);   
	}
	
	return sum;
}

int PickBranch(samplesize,Ttotal,tree)
int samplesize;
double Ttotal;
struct node *tree;
{
	int x;
	double Tsum,Tpick;

	Tsum = 0.0;
	Tpick = Ttotal*unirandom();
	for(x=(2*samplesize-2);x>=samplesize;x--) {
		Tsum += (tree[x].time - (tree+x)->desc1->time); 
		if( Tpick < Tsum ) 
			return ( (tree+x)->desc1->id ); 
		Tsum += (tree[x].time - (tree+x)->desc2->time);   
		if( Tpick < Tsum ) 
			return ( (tree+x)->desc2->id ); 
	}
}

void Mutate(samplesize,site,datamatrix,thisnode)
int samplesize,site;
char **datamatrix;
struct node *thisnode;
{
	if( thisnode->id >= samplesize ) {
		Mutate(samplesize,site,datamatrix,thisnode->desc1);
		Mutate(samplesize,site,datamatrix,thisnode->desc2);
	} else 
		datamatrix[thisnode->id][site] = '1';
}

void MakeData(samplesize,S,datamatrix,Ttotal,tree)
int samplesize,S;
char **datamatrix;
double Ttotal;
struct node *tree;
{
	int x,y,nodenum;

	for(x=0;x<samplesize;x++) {
		for(y=0;y<S;y++) 
			datamatrix[x][y] = '0'; 
		datamatrix[x][S] = '\0';
	}
	for(x=0;x<S;x++) { 
		nodenum = PickBranch(samplesize,Ttotal,tree); 
		Mutate(samplesize,x,datamatrix,tree+nodenum);
	}
}

void PrintData(samplesize,S,datamatrix,fptr)
int samplesize,S;
char **datamatrix;
FILE *fptr;
{
	int x;

	fprintf(fptr," %d %d \n",samplesize,S);
	for(x=0;x<samplesize;x++) {
		fputs(datamatrix[x],fptr);
		fputc('\n',fptr);
	}
	fputc('\n',fptr);
	fflush(fptr);
}

void WriteTreeInfo(samplesize,tree,fptr)
int samplesize;
struct node *tree;
FILE *fptr;
{
	int x;

	for(x=0;x<(2*samplesize - 1);x++) {
		fprintf(fptr,"\t%d",(tree + x)->id);
		fprintf(fptr,"\t%f",(tree + x)->time);
		if( (tree + x)->desc1 != NULL )
			fprintf(fptr,"\t%d",(tree + x)->desc1->id);
		if( (tree + x)->desc2 != NULL )
			fprintf(fptr,"\t%d",(tree + x)->desc2->id);
		fprintf(fptr,"\t%d",(tree + x)->parenttofollow);
		fprintf(fptr,"\t%d",(tree + x)->chromosomeinparent);
		fputc('\n',fptr);
		fflush(fptr);
	}
	fputc('\n',fptr);
	fflush(fptr);
}

void WriteParenthetical(node,sample,fptr)
struct node node;
int sample;
FILE *fptr;
{
	if(node.id>=sample)
		fputc('(',fptr);
	if(node.desc1->id<sample)
		fprintf(fptr,"seq_%1i",node.desc1->id);
	else
		WriteParenthetical(*(node.desc1),sample,fptr);
	fputc(',',fptr);
	if(node.desc2->id<sample)
		fprintf(fptr,"seq_%1i",node.desc2->id);
	else
		WriteParenthetical(*(node.desc2),sample,fptr);
	if(node.id>=sample)
		fputc(')',fptr);
}

/* the stuff above (up to HEY) originated from your PlainCoal/coalesce.c */
/* the three routines below come from ManyDemes/coalmigtest.c */ 
/* but modified to do only what you need now */ 
/* note, site frequencies are unfolded rather than folded */

int Nchoose2(number)
int number;
{
	if(number < 2)
		return 0;
	else
		return ( number*(number-1)/2 );
}

void CountFreqs(samplesize,S,sitefreqs,datamatrix,pip)
int samplesize,S,*sitefreqs;
char **datamatrix;
double *pip;
{
	int x,y,count;

	for(x=1;x<=samplesize-1;x++)
		sitefreqs[x] = 0;
	for(x=0;x<S;x++) {
		count = 0;
		for(y=0;y<samplesize;y++)
			if(datamatrix[y][x] == '1')
				count += 1;
		sitefreqs[count] += 1;
	}
	*pip = 0.0;
	for(y=1;y<=samplesize-1;y++)
		*pip += (double)(y*(samplesize-y)*sitefreqs[y]);
	*pip /= (double)Nchoose2(samplesize);
}

void PrintStats(N,numreps,samplesize,theta,freqsums,sums,sumsqs,fptr)
int N,numreps,samplesize;
double theta,*freqsums,sums[10],sumsqs[10];
FILE *fptr;
{
	int x,y; 
	double nreps;
	
	nreps = (double)numreps;
	fprintf(fptr,"\n");
	fprintf(fptr,"\tN = %d\n",N); 
	fprintf(fptr,"\ttheta = %f\n",theta); 
	fprintf(fptr,"\tsamplesize = %d\n",samplesize); 
	fprintf(fptr,"\tnumreps = %d\n",numreps); 
	fprintf(fptr,"\n");
    fprintf(fptr,"\tnum. sites w/ \t average \t (expected)\n");
	for(y=1;y<=samplesize-1;y++) 
		fprintf(fptr,"\tmutant count %d \t %f \t (%f)\n",y,freqsums[y]/nreps,theta/((double)y));
	fprintf(fptr,"\n");
    fprintf(fptr,"\taverage site frequency spectrum: \n");
	for(y=1;y<=samplesize-1;y++) 
		if(y==1) 
            if(samplesize>2) 
                fprintf(fptr,"\tsitefreqs = {%f,",freqsums[y]/nreps/(sums[1]/nreps));
            else
                fprintf(fptr,"\tsitefreqs = {%f}\n",freqsums[y]/nreps/(sums[1]/nreps));
		else 
			if(y==samplesize-1)
				fprintf(fptr,"%f}\n",freqsums[y]/nreps/(sums[1]/nreps));
			else
				fprintf(fptr,"%f,",freqsums[y]/nreps/(sums[1]/nreps));
	fprintf(fptr,"\n");
	fprintf(fptr,"\taverage Ttotal = %f\n",(sums[0]/nreps)/(2.0*((double)N))); 
	fprintf(fptr,"\tvariance Ttotal = %f\n",((nreps/(nreps-1.0))*(sumsqs[0]/nreps - (sums[0]/nreps)*(sums[0]/nreps)))/(4.0*((double)(N*N))));  
	fprintf(fptr,"\taverage S = %f\n",sums[1]/nreps); 
	fprintf(fptr,"\tvariance S = %f\n",(nreps/(nreps-1.0))*(sumsqs[1]/nreps - (sums[1]/nreps)*(sums[1]/nreps)));  
	fprintf(fptr,"\taverage pi = %f\n",sums[2]/nreps); 
	fprintf(fptr,"\tvariance pi = %f\n",(nreps/(nreps-1.0))*(sumsqs[2]/nreps - (sums[2]/nreps)*(sums[2]/nreps)));  
	fflush(fptr);
}

int main(void) 
{
	int x,y;  /* counters for for loops, etc. */
	int numgens;  /* the number of generations in the pedigree; not global, so every function that uses the pedigree has this as an argument */ 
                  /* note that these generations are numbered/indexed from 0 for the present to numgens-1 for the most ancient. */
	int N;        /* the population size of individuals (num males + num females) under constant model */
	int numreps;  /* number of replicates to do, of various things */
	double tmrca; /* time to the most recent common ancestor of a sample */
	double f;     /* the fraction of the population that is female */
	double sums[10],sumsqs[10];  /* generic vars and arrays to compute means and variances, for testing */
	int samplesize;  /* the number of different individuals to sample; at the moment only following one lineage from each  */
    int *sampleids;  /* the ids of the sampled individuals whose ancestry you are following (again, one lineage from each) */
	time_t first,start,finish; 
	FILE *fptr;
	/* variables below are from coalesce.c */
	int S,S2;  /* numbers of segregating sites in pseudo-datasets */
	char **datamatrix;  /* the pseudo-dataset */
	double theta;  /* 4Nu */ 
	double Ttotal;  /* total length of a gene genealogy */ 
	struct node *tree;  /* the gene genealogy */ 
	/* variables above from coalesce.c */
	/* variables below from coalmigtest.c, or at least needed for tests from there. commented some. */
	int *sitefreqs;  /* the unfolded site frequency counts (for one dataset) */
	double *freqsums;  /* for computing the average of the unfolded site frequency counts over replicate data sets */
	double pi;  /* pi is the average number of pairwise differences */
	
	time(&start); 
	srandom( (unsigned long)start );  /* seed with current time */
	/* srandom( (unsigned long)14 ); */  /* specify a seed */
	first = start;

	fptr = fopen("results","w"); 
	/* fptr = stdout; */
	
	samplesize = 10;   /* specify the sample size */

	if( samplesize > N ) MyError("samplesize > N in main(), but can't do this (yet?)");
	sampleids = (int *)malloc( (size_t)samplesize*sizeof(int) );
	if(sampleids == NULL) MyError("problem getting memory for sampleids in main");
	
	tree = (struct node *)malloc( (size_t)2*samplesize*sizeof(struct node) );
	if(tree == NULL) MyError("problem getting memory for tree in main");
	sitefreqs = (int *)malloc( (size_t)samplesize*sizeof(int) );  /* dumb: you don't ever use the zero cell; site freqs run from 1 to samplesize-1 */
	if(sitefreqs == NULL) MyError("problem getting memory for sitefreqs in main");
	freqsums = (double *)malloc( (size_t)samplesize*sizeof(double) );
	if(freqsums == NULL) MyError("problem getting memory for freqsums in main");
    
	N = 2000;   /* specify the population size */
	numgens = 25*N;  /* set the maximum number of generations allowed in the pedigree; rest as well, for each value of N. */
		/* 30N is a very safe value since it makes the chance of two lines not coalescing in the pedigree Exp[-30N/2N]=Exp[-15]=3x10^-7. */
        /* NOTE: pedigree memory requirement is proportional to N*numgens, and with N=2000 and numgens=50000 it takes about 760MB. */
	f = 0.5;    /* specify the sex ratio */
	theta = 0.5;  /* most of the time you'll probable keep this (4Nu) the same for series of N's, but could do otherwise. */

	/* This block makes one Wright-Fisher pedigree which is then used for every replicate. */ 
	/* Alternatively you can have it make a cyclical Wright-Fisher pedigree (see below). */ 
	/* For each of numreps, it follows all of the samplesize lineages back in time to their most recent common ancestor, */
    /* constructing their gene genealogy, and pseudo-data as needed.  Thus numreps is like the number of loci sequenced. */
	/* At the moment, it just computes the site frequencies, S, Ttotal, and pi, and some variances.  */
	/* If necessary, it loops back around the pedigree-as-cylinder, the way you do it now; see FollowLineagesMakeTree. */
    /* NOTE: numreps independent gene genealogy+pedigree pairs can be generated simply by moving the line: */
    /* MakeWrightFisherPedigree(numgens); (or MakeCyclicalWrightFisherPedigree(numgens);) */
    /* to just inside the x for-loop */
	
    AllocateTopLevel(numgens); 
    SetPopSizesAllocatePedigree(numgens,N,f);
    MakeWrightFisherPedigree(numgens);
    /* MakeCyclicalWrightFisherPedigree(numgens); */
    InitializeTree(samplesize,tree);
    numreps = 100000;
    for(x=0;x<10;x++) {
        sums[x] = 0.0;
        sumsqs[x] = 0.0;
    }
    for(y=1;y<=samplesize-1;y++) 
        freqsums[y] = 0.0;
    time(&start);
    for(x=0;x<numreps;x++) {
        SampleWithoutReplacement(0,popsizes[0][0]+popsizes[0][1]-1,samplesize,sampleids);
        InitializeTreeTips(samplesize,sampleids,tree);
        FollowLineagesMakeTree(numgens,samplesize,tree); 
        Ttotal = CalculateTotalTreeLength(samplesize,tree);
        sums[0] += Ttotal;
        sumsqs[0] += Ttotal*Ttotal;
        S = (int)poissondev(Ttotal*theta/(4.0*(double)N));
        sums[1] += (double)S;
        sumsqs[1] += (double)(S*S);
        if(S > 0) {
            datamatrix = (char **)malloc( (size_t)samplesize*sizeof(char *) );
            if( datamatrix == NULL ) MyError("prob w/datamatrix in main()");
            for(y=0;y<samplesize;y++) {
                datamatrix[y] = (char *)malloc( (size_t)(S+1)*sizeof(char) );
                if( datamatrix[y] == NULL ) MyError("prob w/datamatrix[y] in main()");
            }
            MakeData(samplesize,S,datamatrix,Ttotal,tree);
            CountFreqs(samplesize,S,sitefreqs,datamatrix,&pi); 
            for(y=1;y<=samplesize-1;y++) 
            freqsums[y] += (double)sitefreqs[y];
            for(y=0;y<samplesize;y++) 
                free( (void *)datamatrix[y] );
            free( (void *)datamatrix ); 
        }  else 
            pi = 0.0;
        sums[2] += pi;
        sumsqs[2] += pi*pi;
    }
    time(&finish);
    PrintStats(N,numreps,samplesize,theta,freqsums,sums,sumsqs,fptr);
    FreeAllMemory(numgens);
    PrintMemoryUsage(fptr); 
    WriteTime(start,finish,fptr); 

	free( (void *)tree );
	free( (void *)sitefreqs );
	free( (void *)freqsums );
	
	free( (void *)sampleids );
	
	fclose(fptr); 
	
	return 0;
}
